# mypy: disable-error-code=assignment

"""Class implementation for the vertical bar chart.
"""

from typing import Dict
from typing import List
from typing import Union

from apysc._chart.add_background_mixin import AddBackgroundMixIn
from apysc._chart.add_border_mixin import AddBorderMixIn
from apysc._chart.background_container_mixin import BackgroundContainerMixIn
from apysc._chart.border_container_mixin import BorderContainerMixIn
from apysc._chart.chart_container_mixin import ChartContainerMixIn
from apysc._chart.create_single_column_y_axis_mixin import CreateSingleColumnYAxisMixIn
from apysc._chart.initialize_each_container_mixin import InitializeEachContainerMixIn
from apysc._chart.overall_container_mixin import OverallContainerMixIn
from apysc._chart.set_initial_background_fill_alpha_mixin import (
    SetInitialBackgroundFillAlphaMixIn,
)
from apysc._chart.set_initial_background_fill_color_mixin import (
    SetInitialBackgroundFillColorMixIn,
)
from apysc._chart.set_initial_border_alpha_mixin import SetInitialBorderAlphaMixIn
from apysc._chart.set_initial_border_color_mixin import SetInitialBorderColorMixIn
from apysc._chart.set_initial_border_thickness_mixin import (
    SetInitialBorderThicknessMixIn,
)
from apysc._chart.set_initial_height_mixin import SetInitialHeightMixIn
from apysc._chart.set_initial_horizontal_padding_mixin import (
    SetInitialHorizontalPaddingMixIn,
)
from apysc._chart.set_initial_matrix_data_mixin import SetInitialMatrixDataMixIn
from apysc._chart.set_initial_overall_container_coordinates_mixin import (
    SetInitialOverallContainerCoordinatesMixIn,
)
from apysc._chart.set_initial_vertical_padding_mixin import (
    SetInitialVerticalPaddingMixIn,
)
from apysc._chart.set_initial_width_mixin import SetInitialWidthMixIn
from apysc._chart.set_initial_x_mixin import SetInitialXMixIn
from apysc._chart.set_initial_y_mixin import SetInitialYMixIn
from apysc._chart.x_axis_container_mixin import XAxisContainerMixIn
from apysc._chart.x_axis_settings import XAxisSettings
from apysc._chart.y_axis_container_mixin import YAxisContainerMixIn
from apysc._chart.y_axis_single_column_settings import YAxisSingleColumnSettings
from apysc._color.color import Color
from apysc._color.colorless import COLORLESS
from apysc._color.colors import Colors
from apysc._color.copy_color_if_default_value_specified_mixin import (
    CopyColorIfDefaultValueSpecifiedMixIn,
)
from apysc._type.array import Array
from apysc._type.dictionary import Dictionary
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn

_DataType = Union[
    Array[Dictionary[String, Union[Int, Number, String]]],
    List[Dict[str, Union[int, float, str]]],
]


class VerticalBarChart(
    VariableNameSuffixMixIn,
    SetInitialMatrixDataMixIn,
    SetInitialXMixIn,
    SetInitialYMixIn,
    SetInitialWidthMixIn,
    SetInitialHeightMixIn,
    SetInitialBackgroundFillColorMixIn,
    SetInitialBackgroundFillAlphaMixIn,
    SetInitialBorderColorMixIn,
    SetInitialBorderAlphaMixIn,
    SetInitialBorderThicknessMixIn,
    SetInitialVerticalPaddingMixIn,
    SetInitialHorizontalPaddingMixIn,
    OverallContainerMixIn,
    BackgroundContainerMixIn,
    ChartContainerMixIn,
    XAxisContainerMixIn,
    YAxisContainerMixIn,
    BorderContainerMixIn,
    InitializeEachContainerMixIn,
    AddBackgroundMixIn,
    AddBorderMixIn,
    SetInitialOverallContainerCoordinatesMixIn,
    CreateSingleColumnYAxisMixIn,
    CopyColorIfDefaultValueSpecifiedMixIn,
):
    """
    The class for the vertical bar chart.
    """

    _x_axis_settings: XAxisSettings
    _y_axis_settings: YAxisSingleColumnSettings

    def __init__(
        self,
        *,
        data: _DataType,
        x_axis_settings: XAxisSettings,
        y_axis_settings: YAxisSingleColumnSettings,
        x: Union[float, Number] = 0,
        y: Union[float, Number] = 0,
        width: Union[int, Int] = 640,
        height: Union[int, Int] = 395,
        background_fill_color: Color = Colors.WHITE_FFFFFF,
        background_fill_alpha: Union[float, Number] = 1.0,
        border_color: Color = COLORLESS,
        border_alpha: Union[float, Number] = 1.0,
        border_thickness: Union[int, Int] = 1,
        vertical_padding: Union[int, Int] = 10,
        horizontal_padding: Union[int, Int] = 10,
        variable_name_suffix: str = "",
    ) -> None:
        """
        Create a vertical bar chart instance.

        Parameters
        ----------
        data : Union[Array[Dictionary[String, Union[Int, Number, String]]], List[Dict[str, Union[int, float, str]]]]  # noqa
            A data array, which contains a 1-dimensional string key dictionary.
            A list of dictionaries or an `ap.Array` of `ap.Dictionary` values
            are acceptable.
            E.g., `[{"column_name_1": 10, "column_name_2"}]`
        x_axis_settings : XAxisSettings
            An x-axis setting.
        y_axis_settings : YAxisSingleColumnSettings
            A y-axis setting.
        x : Union[Number]
            A chart's x-coordinate.
        y : Union[float, Number]
            A chart's y-coordinate.
        width : Union[int, Int], default 640
            A chart's width.
        height : Union[int, Int], default 395
            A chart's height.
        background_fill_color : Color, default Colors.WHITE_FFFFFF
            A chart's background fill-color.
        background_fill_alpha : Union[float, Number], default 1.0
            A chart's background fill-alpha.
        border_color : Color, default COLORLESS
            A chart's border color.
        border_alpha : Union[float, Number], default 1.0
            A chart's border alpha.
        border_thickness : Union[int, Int], default 1
            A chart's border thickness.
        vertical_padding : Union[int, Int], default 10
            A chart's vertical padding between borders and contents.
        horizontal_padding : Union[int, Int], default 10
            A chart's horizontal padding between borders and contents.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        self._variable_name_suffix = variable_name_suffix
        self._set_initial_matrix_data(
            data=data, variable_name_suffix=variable_name_suffix
        )
        self._x_axis_settings = x_axis_settings
        self._y_axis_settings = y_axis_settings
        self._set_initial_x(x=x, variable_name_suffix=variable_name_suffix)
        self._set_initial_y(y=y, variable_name_suffix=variable_name_suffix)
        self._set_initial_width(width=width, variable_name_suffix=variable_name_suffix)
        self._set_initial_height(
            height=height, variable_name_suffix=variable_name_suffix
        )
        background_fill_color = self._copy_color_if_default_value_specified(
            color=background_fill_color,
            default_color=Colors.WHITE_FFFFFF,
        )
        self._set_initial_background_fill_color(
            background_fill_color=background_fill_color,
        )
        self._set_initial_background_fill_alpha(
            background_fill_alpha=background_fill_alpha,
            variable_name_suffix=variable_name_suffix,
        )
        self._set_initial_border_color(border_color=border_color)
        self._set_initial_border_alpha(
            border_alpha=border_alpha, variable_name_suffix=variable_name_suffix
        )
        self._set_initial_border_thickness(
            border_thickness=border_thickness, variable_name_suffix=variable_name_suffix
        )
        self._set_initial_vertical_padding(
            vertical_padding=vertical_padding, variable_name_suffix=variable_name_suffix
        )
        self._set_initial_horizontal_padding(
            horizontal_padding=horizontal_padding,
            variable_name_suffix=variable_name_suffix,
        )
        self._initialize_each_container(variable_name_suffix=variable_name_suffix)
        self._add_background(
            background_container=self._background_container,
            width=self._width,
            height=self._height,
            background_fill_color=self._background_fill_color,
            background_fill_alpha=self._background_fill_alpha,
            variable_name_suffix=variable_name_suffix,
        )
        self._add_border(
            border_container=self._border_container,
            width=self._width,
            height=self._height,
            border_color=self._border_color,
            border_alpha=self._border_alpha,
            border_thickness=self._border_thickness,
            variable_name_suffix=variable_name_suffix,
        )
        self._set_initial_overall_container_coordinates(
            overall_container=self._overall_container,
            x=self._x,
            y=self._y,
        )
        self._create_y_axis(
            data=self._matrix_data,
            y_axis_container=self._y_axis_container,
            chart_height=self._height,
            x_axis_settings=self._x_axis_settings,
            y_axis_settings=self._y_axis_settings,
            vertical_padding=self._vertical_padding,
            horizontal_padding=self._horizontal_padding,
            variable_name_suffix=variable_name_suffix,
        )



"""The class implementation for the `closed_caption_disabled` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialClosedCaptionDisabledIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `closed_caption_disabled` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><rect fill="none" height="24" width="24"/><path d="M6.83,4H19c1.1,0,2,0.9,2,2v12c0,0.05-0.01,0.1-0.02,0.16l-3.38-3.38C17.84,14.59,18,14.32,18,14v-1h-1.5v0.5h-0.17 l-1.83-1.83V10.5h2V11H18v-1c0-0.55-0.45-1-1-1h-3c-0.55,0-1,0.45-1,1v0.17L6.83,4z M19.78,22.61L17.17,20H5c-1.11,0-2-0.9-2-2V6 c0-0.05,0.02-0.1,0.02-0.15L1.39,4.22l1.41-1.41l18.38,18.38L19.78,22.61z M11,13.83L10.17,13H9.5v0.5h-2v-3h0.17L6.4,9.22 C6.16,9.41,6,9.68,6,10v4c0,0.55,0.45,1,1,1h3c0.55,0,1-0.45,1-1V13.83z"/></svg>'  # noqa



"""The class implementation for the outlined `live_help` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialLiveHelpOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `live_help` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M19 2H5c-1.11 0-2 .9-2 2v14c0 1.1.89 2 2 2h4l3 3 3-3h4c1.1 0 2-.9 2-2V4c0-1.1-.9-2-2-2zm0 16h-4.83l-.59.59L12 20.17l-1.59-1.59-.58-.58H5V4h14v14zm-8-3h2v2h-2zm1-8c1.1 0 2 .9 2 2 0 2-3 1.75-3 5h2c0-2.25 3-2.5 3-5 0-2.21-1.79-4-4-4S8 6.79 8 9h2c0-1.1.9-2 2-2z"/></svg>'  # noqa



"""The class implementation for the `accessible` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAccessibleIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `accessible` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><circle cx="12" cy="4" r="2"/><path d="M19 13v-2c-1.54.02-3.09-.75-4.07-1.83l-1.29-1.43c-.17-.19-.38-.34-.61-.45-.01 0-.01-.01-.02-.01H13c-.35-.2-.75-.3-1.19-.26C10.76 7.11 10 8.04 10 9.09V15c0 1.1.9 2 2 2h5v5h2v-5.5c0-1.1-.9-2-2-2h-3v-3.45c1.29 1.07 3.25 1.94 5 1.95zm-6.17 5c-.41 1.16-1.52 2-2.83 2-1.66 0-3-1.34-3-3 0-1.31.84-2.41 2-2.83V12.1c-2.28.46-4 2.48-4 4.9 0 2.76 2.24 5 5 5 2.42 0 4.44-1.72 4.9-4h-2.07z"/></svg>'  # noqa



"""The class implementation for the `person_search` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPersonSearchIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `person_search` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><circle cx="10" cy="8" r="4"/><path d="M10.35,14.01C7.62,13.91,2,15.27,2,18v2h9.54C9.07,17.24,10.31,14.11,10.35,14.01z"/><path d="M19.43,18.02C19.79,17.43,20,16.74,20,16c0-2.21-1.79-4-4-4s-4,1.79-4,4c0,2.21,1.79,4,4,4c0.74,0,1.43-0.22,2.02-0.57 L20.59,22L22,20.59L19.43,18.02z M16,18c-1.1,0-2-0.9-2-2c0-1.1,0.9-2,2-2s2,0.9,2,2C18,17.1,17.1,18,16,18z"/></g></g></svg>'  # noqa



"""The class implementation for the `stop_screen_share` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialStopScreenShareIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `stop_screen_share` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M21.22 18.02l2 2H24v-2h-2.78zm.77-2l.01-10c0-1.11-.9-2-2-2H7.22l5.23 5.23c.18-.04.36-.07.55-.1V7.02l4 3.73-1.58 1.47 5.54 5.54c.61-.33 1.03-.99 1.03-1.74zM2.39 1.73L1.11 3l1.54 1.54c-.4.36-.65.89-.65 1.48v10c0 1.1.89 2 2 2H0v2h18.13l2.71 2.71 1.27-1.27L2.39 1.73zM7 15.02c.31-1.48.92-2.95 2.07-4.06l1.59 1.59c-1.54.38-2.7 1.18-3.66 2.47z"/></svg>'  # noqa



"""The class implementation for the `search_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialSearchOffIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `search_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><path d="M15.5,14h-0.79l-0.28-0.27C15.41,12.59,16,11.11,16,9.5C16,5.91,13.09,3,9.5,3C6.08,3,3.28,5.64,3.03,9h2.02 C5.3,6.75,7.18,5,9.5,5C11.99,5,14,7.01,14,9.5S11.99,14,9.5,14c-0.17,0-0.33-0.03-0.5-0.05v2.02C9.17,15.99,9.33,16,9.5,16 c1.61,0,3.09-0.59,4.23-1.57L14,14.71v0.79l5,4.99L20.49,19L15.5,14z"/><polygon points="6.47,10.82 4,13.29 1.53,10.82 0.82,11.53 3.29,14 0.82,16.47 1.53,17.18 4,14.71 6.47,17.18 7.18,16.47 4.71,14 7.18,11.53"/></g></g></svg>'  # noqa



"""The class implementation for the outlined `attribution` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAttributionOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `attribution` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M12 8.5c-.91 0-2.75.46-2.75 1.38v4.62h1.5V19h2.5v-4.5h1.5V9.88c0-.91-1.84-1.38-2.75-1.38zM12 2C6.47 2 2 6.47 2 12s4.47 10 10 10 10-4.48 10-10S17.52 2 12 2zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z"/><circle cx="12" cy="6.5" r="1.5"/></svg>'  # noqa



"""The class implementation for the outlined `sort_by_alpha` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialSortByAlphaOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `sort_by_alpha` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M14.94 4.66h-4.72l2.36-2.36 2.36 2.36zm-4.69 14.71h4.66l-2.33 2.33-2.33-2.33zM6.1 6.27L1.6 17.73h1.84l.92-2.45h5.11l.92 2.45h1.84L7.74 6.27H6.1zm-1.13 7.37l1.94-5.18 1.94 5.18H4.97zm10.76 2.5h6.12v1.59h-8.53v-1.29l5.92-8.56h-5.88v-1.6h8.3v1.26l-5.93 8.6z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: string_length.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# String class length property": "# String クラスの length 属性",
    ##################################################
    "This page explains the `String` class `length` property.": "このページでは`String`クラスの`length`属性について説明します。",  # noqa
    ##################################################
    "## What property is this?": "## 属性の概要",
    ##################################################
    "The `length` property returns the number of characters.": "`length`属性は文字数を返却します。",
    ##################################################
    "For example, the `ABCDEF` string returns 6, and the `あいうえお` string returns 5.": "例えば`ABCDEF`という文字列では6が返却され、`あいうえお`という文字列では5が返却されます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `length` property returns an `Int` value as follows:": "`length`属性では以下のように`Int`の値を返却します。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nstring: ap.String = ap.String("ABCDEF")\nlength: ap.Int = string.length\nap.assert_equal(length, 6)\n\nap.save_overall_html(dest_dir_path="string_length_basic_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nstring: ap.String = ap.String("ABCDEF")\nlength: ap.Int = string.length\nap.assert_equal(length, 6)\n\nap.save_overall_html(dest_dir_path="string_length_basic_usage_1/")\n```',  # noqa
    ##################################################
    "## Notes of the emoji": "## 絵文字に関する特記事項",
    ##################################################
    "This property returns an unexpected characters length when the string is an emoji character: since this property counts Unicode code points.": "この属性はUnicodeのコードポイント数をカウントしているため、絵文字を対象とした場合に想定外の文字列を返却することがあります。",  # noqa
    ##################################################
    "Most of the emoji characters behave as expected length, as follows:": "大半の絵文字は以下のように想定通りの文字数として振る舞います。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nstring: ap.String = ap.String("🎉")\nap.assert_equal(string.length, 1)\n\nstring = ap.String("🥳🌟🍻")\nap.assert_equal(string.length, 3)\n\nap.save_overall_html(dest_dir_path="string_length_notes_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nstring: ap.String = ap.String("🎉")\nap.assert_equal(string.length, 1)\n\nstring = ap.String("🥳🌟🍻")\nap.assert_equal(string.length, 3)\n\nap.save_overall_html(dest_dir_path="string_length_notes_1/")\n```',  # noqa
    ##################################################
    "However, in some emojis that have multiple code points, this property returns an unexpected length of characters (this behavior is the same as the Python):": "しかしながら複数のコードポイントを持つ絵文字に関してはこの属性は想定外の文字数を返却します（これはPythonと同じような挙動をします）:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nassert len("👨‍👩‍👦") == 5\n\nstring: ap.String = ap.String("👨‍👩‍👦")\nap.assert_equal(string.length, 5)\n\nap.save_overall_html(dest_dir_path="string_length_notes_2/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nassert len("👨‍👩‍👦") == 5\n\nstring: ap.String = ap.String("👨‍👩‍👦")\nap.assert_equal(string.length, 5)\n\nap.save_overall_html(dest_dir_path="string_length_notes_2/")\n```',  # noqa
    ##################################################
    "## length property API": "## length 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a character length (number).<hr>": "文字の長さ（文字数）を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `characters_length`: Int": "- `characters_length`: Int",
    ##################################################
    "  - A character length (number).": "  - 文字の長さ（文字数）。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> string: ap.String = ap.String("Hello")\n>>> string.length\nInt(5)\n```': '```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> string: ap.String = ap.String("Hello")\n>>> string.length\nInt(5)\n```',  # noqa
}



"""The class implementation for the outlined `eco` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialEcoOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `eco` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><path d="M6.05,8.05c-2.73,2.73-2.73,7.17,0,9.9C7.42,19.32,9.21,20,11,20s3.58-0.68,4.95-2.05C19.43,14.47,20,4,20,4 S9.53,4.57,6.05,8.05z M14.54,16.54C13.59,17.48,12.34,18,11,18c-0.89,0-1.73-0.25-2.48-0.68c0.92-2.88,2.62-5.41,4.88-7.32 c-2.63,1.36-4.84,3.46-6.37,6c-1.48-1.96-1.35-4.75,0.44-6.54C9.21,7.72,14.04,6.65,17.8,6.2C17.35,9.96,16.28,14.79,14.54,16.54z"/></g></svg>'  # noqa



"""The class implementation for the outlined `all_inbox` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAllInboxOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `all_inbox` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M19 3H5c-1.1 0-2 .9-2 2v7c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2zM5 10h3.13c.21.78.67 1.47 1.27 2H5v-2zm14 2h-4.4c.6-.53 1.06-1.22 1.27-2H19v2zm0-4h-5v1c0 1.07-.93 2-2 2s-2-.93-2-2V8H5V5h14v3zm-2 7h-3v1c0 .47-.19.9-.48 1.25-.37.45-.92.75-1.52.75s-1.15-.3-1.52-.75c-.29-.35-.48-.78-.48-1.25v-1H3v4c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2v-4h-4zM5 17h3.13c.02.09.06.17.09.25.24.68.65 1.28 1.18 1.75H5v-2zm14 2h-4.4c.54-.47.95-1.07 1.18-1.75.03-.08.07-.16.09-.25H19v2z"/></svg>'  # noqa



"""The class implementation for the outlined `location_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialLocationOffOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `location_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M12 4c2.76 0 5 2.24 5 5 0 1.06-.39 2.32-1 3.62l1.49 1.49C18.37 12.36 19 10.57 19 9c0-3.87-3.13-7-7-7-1.84 0-3.5.71-4.75 1.86l1.43 1.43C9.56 4.5 10.72 4 12 4zm0 2.5c-.59 0-1.13.21-1.56.56l3.5 3.5c.35-.43.56-.97.56-1.56 0-1.38-1.12-2.5-2.5-2.5zM3.41 2.86L2 4.27l3.18 3.18C5.07 7.95 5 8.47 5 9c0 5.25 7 13 7 13s1.67-1.85 3.38-4.35L18.73 21l1.41-1.41L3.41 2.86zM12 18.88c-2.01-2.58-4.8-6.74-4.98-9.59l6.92 6.92c-.65.98-1.33 1.89-1.94 2.67z"/></svg>'  # noqa



"""The class implementation for the `hourglass_disabled` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialHourglassDisabledIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `hourglass_disabled` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><polygon points="8,4 16,4 16,7.5 13.16,10.34 14.41,11.59 18,8.01 17.99,8 18,8 18,2 6,2 6,3.17 8,5.17"/><path d="M2.1,2.1L0.69,3.51l8.9,8.9L6,16l0.01,0.01H6V22h12v-1.17l2.49,2.49l1.41-1.41L2.1,2.1z M16,20H8v-3.5l2.84-2.84L16,18.83 V20z"/></g></g></svg>'  # noqa



"""The class implementation for the `auto_delete` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAutoDeleteIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `auto_delete` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><polygon points="15,2 11.5,2 10.5,1 5.5,1 4.5,2 1,2 1,4 15,4"/><path d="M16,9c-0.7,0-1.37,0.1-2,0.29V5H2v12c0,1.1,0.9,2,2,2h5.68c1.12,2.36,3.53,4,6.32,4c3.87,0,7-3.13,7-7 C23,12.13,19.87,9,16,9z M16,21c-2.76,0-5-2.24-5-5s2.24-5,5-5s5,2.24,5,5S18.76,21,16,21z"/><polygon points="16.5,12 15,12 15,17 18.6,19.1 19.4,17.9 16.5,16.2"/></g></g></svg>'  # noqa



"""The mix-in class implementation for the `_set_initial_horizontal_padding` method.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._validation import arg_validation_decos


class SetInitialHorizontalPaddingMixIn:
    _horizontal_padding: Int

    @final
    @arg_validation_decos.is_integer(arg_position_index=1, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_horizontal_padding(
        self,
        *,
        horizontal_padding: Union[int, Int],
        variable_name_suffix: str = "",
    ) -> None:
        """
        Set a chart's initial horizontal padding.

        Parameters
        ----------
        horizontal_padding : Union[int, Int]
            A chart's horizontal padding between borders and contents.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        if not isinstance(horizontal_padding, Int):
            horizontal_padding_: Int = Int(
                horizontal_padding, variable_name_suffix=variable_name_suffix
            )
        else:
            horizontal_padding_ = horizontal_padding
        self._horizontal_padding = horizontal_padding_



"""The width geometry mix-in implementation for the `RectangleGeom`.
"""

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int


class RectangleGeomWidthMixIn:
    _width: Int

    @property
    @add_debug_info_setting(module_name=__name__)
    def width(self) -> Int:
        """
        Get the rectangle geometry width.

        Returns
        -------
        width : Int
            The rectangle geometry width.
        """
        return self._width



"""Implementations of the Line class.
"""

from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._color.color import Color
from apysc._color.colorless import COLORLESS
from apysc._display import graphics
from apysc._display.add_to_parent_mixin import AddToParentMixIn
from apysc._display.append_line_alpha_attr_expression_mixin import (
    AppendLineAlphaAttrExpressionMixIn,
)
from apysc._display.append_line_cap_attr_expression_mixin import (
    AppendLineCapAttrExpressionMixIn,
)
from apysc._display.append_line_color_attr_expression_mixin import (
    AppendLineColorAttrExpressionMixIn,
)
from apysc._display.append_line_thickness_attr_expression_mixin import (
    AppendLineThicknessAttrExpressionMixIn,
)
from apysc._display.append_x_attr_expression_mixin import AppendXAttrExpressionMixIn
from apysc._display.append_y_attr_expression_mixin import AppendYAttrExpressionMixIn
from apysc._display.child_mixin import ChildMixIn
from apysc._display.css_mixin import CssMixIn
from apysc._display.flip_x_mixin import FlipXMixIn
from apysc._display.flip_y_mixin import FlipYMixIn
from apysc._display.get_bounds_mixin import GetBoundsMixIn
from apysc._display.graphics_base import GraphicsBase
from apysc._display.line_alpha_mixin import LineAlphaMixIn
from apysc._display.line_caps import LineCaps
from apysc._display.line_color_mixin import LineColorMixIn
from apysc._display.line_dash_dot_setting import LineDashDotSetting
from apysc._display.line_dash_dot_setting_mixin import LineDashDotSettingMixIn
from apysc._display.line_dash_setting import LineDashSetting
from apysc._display.line_dash_setting_mixin import LineDashSettingMixIn
from apysc._display.line_dot_setting import LineDotSetting
from apysc._display.line_dot_setting_mixin import LineDotSettingMixIn
from apysc._display.line_round_dot_setting import LineRoundDotSetting
from apysc._display.line_round_dot_setting_mixin import LineRoundDotSettingMixIn
from apysc._display.rotation_around_center_mixin import RotationAroundCenterMixIn
from apysc._display.rotation_around_point_mixin import RotationAroundPointMixIn
from apysc._display.scale_x_from_center_mixin import ScaleXFromCenterMixIn
from apysc._display.scale_x_from_point_mixin import ScaleXFromPointMixIn
from apysc._display.scale_y_from_center_mixin import ScaleYFromCenterMixIn
from apysc._display.scale_y_from_point_mixin import ScaleYFromPointMixIn
from apysc._display.set_overflow_visible_setting_mixin import (
    SetOverflowVisibleSettingMixIn,
)
from apysc._display.skew_x_mixin import SkewXMixIn
from apysc._display.skew_y_mixin import SkewYMixIn
from apysc._display.x_mixin import XMixIn
from apysc._display.y_mixin import YMixIn
from apysc._geom import point2d
from apysc._html.debug_mode import add_debug_info_setting
from apysc._loop.initialize_with_base_value_interface import (
    InitializeWithBaseValueInterface,
)
from apysc._mask.svg_mask_mixin import SvgMaskMixIn
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.repr_interface import ReprInterface
from apysc._type.string import String
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class Line(
    ReprInterface,
    XMixIn,
    AppendXAttrExpressionMixIn,
    YMixIn,
    AppendYAttrExpressionMixIn,
    SetOverflowVisibleSettingMixIn,
    CssMixIn,
    GraphicsBase,
    RotationAroundCenterMixIn,
    RotationAroundPointMixIn,
    ScaleXFromCenterMixIn,
    ScaleYFromCenterMixIn,
    ScaleXFromPointMixIn,
    ScaleYFromPointMixIn,
    FlipXMixIn,
    FlipYMixIn,
    SkewXMixIn,
    SkewYMixIn,
    LineColorMixIn,
    AppendLineColorAttrExpressionMixIn,
    LineAlphaMixIn,
    AppendLineAlphaAttrExpressionMixIn,
    AppendLineThicknessAttrExpressionMixIn,
    AppendLineCapAttrExpressionMixIn,
    LineDotSettingMixIn,
    LineDashSettingMixIn,
    LineRoundDotSettingMixIn,
    LineDashDotSettingMixIn,
    GetBoundsMixIn,
    VariableNameSuffixMixIn,
    InitializeWithBaseValueInterface,
    AddToParentMixIn,
    SvgMaskMixIn,
):
    """
    The line vector graphics class.

    References
    ----------
    - Line class
        - https://simon-ritchie.github.io/apysc/en/line.html
    - Graphics draw_line interface
        - https://simon-ritchie.github.io/apysc/en/graphics_draw_line.html  # noqa
    - Graphics draw_dotted_line interface
        - https://simon-ritchie.github.io/apysc/en/graphics_draw_dotted_line.html  # noqa
    - Graphics draw_dashed_line interface
        - https://simon-ritchie.github.io/apysc/en/graphics_draw_dashed_line.html  # noqa
    - Graphics draw_round_dotted_line interface
        - https://simon-ritchie.github.io/apysc/en/graphics_draw_round_dotted_line.html  # noqa
    - Graphics draw_dash_dotted_line interface
        - https://simon-ritchie.github.io/apysc/en/graphics_draw_dash_dotted_line.html # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=5)
    >>> line: ap.Line = sprite.graphics.draw_line(
    ...     x_start=50, y_start=50, x_end=150, y_end=50
    ... )
    >>> line.line_color
    Color("#ffffff")

    >>> line.line_thickness
    Int(5)
    """

    _start_point: "point2d.Point2D"
    _end_point: "point2d.Point2D"

    # self
    @arg_validation_decos.multiple_line_settings_are_not_set(arg_position_index=0)
    # start_point
    @arg_validation_decos.is_point_2d(arg_position_index=1)
    # end_point
    @arg_validation_decos.is_point_2d(arg_position_index=2)
    # line_color
    @arg_validation_decos.is_color(arg_position_index=3, optional=False)
    # line_alpha
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=4, optional=False)
    # line_thickness
    @arg_validation_decos.is_integer(arg_position_index=5, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=5, optional=False)
    # line_cap
    @arg_validation_decos.is_line_cap(arg_position_index=6, optional=True)
    # line_dot_setting
    @arg_validation_decos.is_line_dot_setting(arg_position_index=7)
    # line_dash_setting
    @arg_validation_decos.is_line_dash_setting(arg_position_index=8)
    # line_round_dot_setting
    @arg_validation_decos.is_line_round_dot_setting(arg_position_index=9)
    # line_dash_dot_setting
    @arg_validation_decos.is_line_dash_dot_setting(arg_position_index=10)
    # parent
    @arg_validation_decos.is_display_object_container(
        arg_position_index=11, optional=True
    )
    # variable_name_suffix
    @arg_validation_decos.is_builtin_string(arg_position_index=12, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        *,
        start_point: "point2d.Point2D",
        end_point: "point2d.Point2D",
        line_color: Color = COLORLESS,
        line_alpha: Union[float, Number] = 1.0,
        line_thickness: Union[int, Int] = 1,
        line_cap: Optional[Union[String, LineCaps]] = None,
        line_dot_setting: Optional[LineDotSetting] = None,
        line_dash_setting: Optional[LineDashSetting] = None,
        line_round_dot_setting: Optional[LineRoundDotSetting] = None,
        line_dash_dot_setting: Optional[LineDashDotSetting] = None,
        parent: Optional[ChildMixIn] = None,
        variable_name_suffix: str = "",
    ) -> None:
        """
        Create a line vector graphic.

        Parameters
        ----------
        start_point : Points2D
            Line start point.
        end_point : Points2D
            Line end point.
        line_color : Color, default COLORLESS
            A line-color to set.
        line_alpha : float or Number, default 1.0
            A line-alpha to set.
        line_thickness : int or Int, default 1
            A line-thickness (line-width) to set.
        line_cap : String or LineCaps or None, default None
            A line-cap setting to set.
        line_dot_setting : LineDotSetting or None, default None
            A dot setting to set.
        line_dash_setting : LineDashSetting or None, default None
            A dash setting to set.
        line_round_dot_setting : LineRoundDotSetting or None, default None
            A round-dot setting to set.
        line_dash_dot_setting : LineDashDotSetting or None, default None
            A dash-dot (1-dot chain) setting to set.
        parent : ChildMixIn or None, default None
            A parent instance to add this instance.
            If the specified value is None, this interface uses
            a stage instance.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        References
        ----------
        - Line class
            - https://simon-ritchie.github.io/apysc/en/line.html

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> line: ap.Line = ap.Line(
        ...     start_point=ap.Point2D(x=50, y=50),
        ...     end_point=ap.Point2D(x=150, y=50),
        ...     line_color=ap.Color("#ffffff"),
        ...     line_thickness=3,
        ... )
        >>> line.line_color
        Color("#ffffff")
        >>> line.line_thickness
        Int(3)
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names

        self._variable_name_suffix = variable_name_suffix
        variable_name: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.LINE
        )
        self.variable_name = variable_name
        self._set_initial_basic_values(
            fill_color=COLORLESS,
            fill_alpha=1,
            line_color=line_color,
            line_thickness=line_thickness,
            line_alpha=line_alpha,
            line_cap=line_cap,
            line_joints=None,
        )
        self._start_point = start_point
        self._end_point = end_point
        self._append_constructor_expression()
        self._set_line_setting_if_not_none_value_exists(
            line_dot_setting=line_dot_setting,
            line_dash_setting=line_dash_setting,
            line_round_dot_setting=line_round_dot_setting,
            line_dash_dot_setting=line_dash_dot_setting,
        )
        self._set_initial_x_and_y_with_minimum_point()
        super(Line, self).__init__(variable_name=variable_name)
        self._set_overflow_visible_setting()
        self._add_to_parent(parent=parent)

    @final
    def _set_initial_x_and_y_with_minimum_point(self) -> None:
        """
        Set initial x and y properties coordinate with
        a minimum point.
        """
        min_x: float = min(self._start_point._x._value, self._end_point._x._value)
        min_y: float = min(self._start_point._y._value, self._end_point._y._value)
        self._x = Number(min_x)
        self._y = Number(min_y)

    @classmethod
    @final
    def _create_with_graphics(
        cls,
        *,
        graphics: "graphics.Graphics",
        start_point: "point2d.Point2D",
        end_point: "point2d.Point2D",
        variable_name_suffix: str = "",
    ) -> "Line":
        """
        Create a line instance with the instance of
        specified graphics.

        Parameters
        ----------
        graphics : Graphics
            Graphics instance to link this instance.
        start_point : Points2D
            Line start point.
        end_point : Points2D
            Line end point.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        line : Line
            A created line instance.
        """
        line: Line = Line(
            start_point=start_point,
            end_point=end_point,
            line_color=graphics._line_color,
            line_alpha=graphics._line_alpha,
            line_thickness=graphics._line_thickness,
            line_cap=graphics._line_cap,
            line_dot_setting=graphics._line_dot_setting,
            line_dash_setting=graphics._line_dash_setting,
            line_round_dot_setting=graphics._line_round_dot_setting,
            line_dash_dot_setting=graphics._line_dash_dot_setting,
            parent=graphics,
            variable_name_suffix=variable_name_suffix,
        )
        return line

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_constructor_expression(self) -> None:
        """
        Append a constructor expression.
        """
        from apysc._display.stage import Stage
        from apysc._display.stage import get_stage
        from apysc._expression import expression_data_util

        INDENT_NUM: int = 2
        stage: Stage = get_stage()
        points_str: str = self._make_points_expression()
        expression: str = (
            f"var {self.variable_name} = {stage.variable_name}"
            f"\n  .line({points_str})"
            "\n  .attr({"
        )
        expression = self._append_line_color_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_line_thickness_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_line_alpha_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_line_cap_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_x_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression = self._append_y_attr_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        expression += "\n  });"
        expression_data_util.append_js_expression(expression=expression)

    @final
    def _make_points_expression(self) -> str:
        """
        Make line start and end expression str.

        Returns
        -------
        expression : str
            Each point expression.
        """
        from apysc._geom.point2d import Point2D

        start_point: Point2D = self._start_point
        end_point: Point2D = self._end_point
        expression: str = (
            f"{start_point.x.variable_name}, "
            f"{start_point.y.variable_name}, "
            f"{end_point.x.variable_name}, "
            f"{end_point.y.variable_name}"
        )
        return expression

    def __repr__(self) -> str:
        """
        Get a string representation of this instance (for the sake of
        debugging).

        Returns
        -------
        repr_str : str
            Type name and variable name will be set
            (e.g., `Line("<variable_name>")`).
        """
        repr_str: str = f'{Line.__name__}("{self.variable_name}")'
        return repr_str

    @classmethod
    @final
    def _initialize_with_base_value(cls) -> "Line":
        """
        Initialize this class with a base value(s).

        Returns
        -------
        line : Line
            An initialized line instance.
        """
        from apysc._type.boolean import Boolean

        line: Line = Line(
            start_point=point2d.Point2D(x=-2, y=-2),
            end_point=point2d.Point2D(x=-1, y=-1),
        )
        line.visible = Boolean(False)
        return line



"""The material color names definitions.
"""

from typing import Literal

MaterialColorNames = Literal[
    "primary",
    "on_primary",
    "secondary",
    "on_secondary",
    "error",
    "on_error",
    "surface",
    "on_surface",
    "primary_container",
    "on_primary_container",
    "secondary_container",
    "on_secondary_container",
    "tertiary",
    "on_tertiary",
    "tertiary_container",
    "on_tertiary_container",
    "error_container",
    "on_error_container",
    "outline",
    "outline_variant",
]



"""The height geometry mix-in implementation for the `RectangleGeom`.
"""

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int


class RectangleGeomHeightMixIn:
    _height: Int

    @property
    @add_debug_info_setting(module_name=__name__)
    def height(self) -> Int:
        """
        Get the rectangle geometry height.

        Returns
        -------
        height : Int
            The rectangle geometry height.
        """
        return self._height



"""Class implementation for the attribute linking mix-in.

This mix-in updates an old property value to achieve
consistency in the handler functions.
"""

from typing import Dict
from typing import List
from typing import Union

from typing_extensions import final

from apysc._color.color import Color
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.boolean import Boolean
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String

_AttrName = str
_Attr = Union[Int, Number, String, Boolean, Color]


class AttrLinkingMixIn:
    _attr_linking_stack: Dict[_AttrName, List[_Attr]]

    @final
    @add_debug_info_setting(module_name=__name__)
    def _initialize_attr_linking_stack(self, *, attr_name: str) -> None:
        """
        Initialize the _attr_linking_stack attribute if
        this instance does not initialize it yet.

        Parameters
        ----------
        attr_name : str
            Target attribute name.
        """
        if not hasattr(self, "_attr_linking_stack"):
            self._attr_linking_stack = {}
        if attr_name in self._attr_linking_stack:
            return
        self._attr_linking_stack[attr_name] = []

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_attr_to_linking_stack(self, *, attr: _Attr, attr_name: str) -> None:
        """
        Append an attribute to the linking attribute stack.

        Parameters
        ----------
        attr : Int or Number or String or Boolean
            Target attribute to be appended.
        attr_name : str
            Target attribute name.
        """
        self._initialize_attr_linking_stack(attr_name=attr_name)
        if self._is_target_attr_already_linked(attr=attr, attr_name=attr_name):
            return
        self._attr_linking_stack[attr_name].append(attr)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _is_target_attr_already_linked(self, *, attr: _Attr, attr_name: str) -> bool:
        """
        Get a boolean value whether this instance already
        appends a specified attribute to the linking
        attribute stack.

        Parameters
        ----------
        attr : Int or Number or String or Boolean
            Target attribute to be appended.
        attr_name : str
            Target attribute name.

        Returns
        -------
        result : bool
            If this instance already appends a specified
            attribute to the linking stack, this interface
            returns True.
        """

        self._initialize_attr_linking_stack(attr_name=attr_name)
        attr_variable_name: str = _get_variable_name_from_attr(attr=attr)
        for in_stack_value in self._attr_linking_stack[attr_name]:
            in_stack_value_variable_name: str = _get_variable_name_from_attr(
                attr=in_stack_value
            )
            if in_stack_value_variable_name == attr_variable_name:
                return True
        return False

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_applying_new_attr_val_exp(
        self, *, new_attr: _Attr, attr_name: str
    ) -> None:
        """
        Append the expression of applying a new attribute
        value to each stacked value.

        Parameters
        ----------
        new_attr : Int or Number or String or Boolean
            New attribute value.

        attr_name : str
            Target attribute name.
        """
        from apysc._expression import expression_data_util

        self._initialize_attr_linking_stack(attr_name=attr_name)
        if not self._attr_linking_stack:
            return
        new_attr_name: str = _get_variable_name_from_attr(attr=new_attr)
        expression: str = ""
        for stacked_value in self._attr_linking_stack[attr_name]:
            stacked_value_variable_name: str = _get_variable_name_from_attr(
                attr=stacked_value
            )
            if stacked_value_variable_name == new_attr_name:
                continue
            if expression != "":
                expression += "\n"
            expression += f"{stacked_value_variable_name} = {new_attr_name};"
        expression_data_util.append_js_expression(expression=expression)


def _get_variable_name_from_attr(*, attr: _Attr) -> str:
    """
    Get a variable name from a specified attribute.

    Parameters
    ----------
    attr : _Attr
        An attribute.

    Returns
    -------
    variable_name : str
        A specified attribute's variable name.
    """
    from apysc._type.variable_name_mixin import VariableNameMixIn

    if isinstance(attr, Color):
        return attr._value.variable_name
    if isinstance(attr, VariableNameMixIn):
        return attr.variable_name
    return ""



"""The loop implementation class for the `ap.Array` indices.
"""

from typing import Any
from typing import Dict
from typing import Optional

from typing_extensions import final

from apysc._expression.get_last_scope_interface import GetLastScopeInterface
from apysc._expression.indent_num import Indent
from apysc._expression.last_scope import LastScope
from apysc._html.debug_mode import add_debug_info_setting
from apysc._loop.for_loop_exit_mixin import ForLoopExitMixIn
from apysc._type.array import Array
from apysc._type.initialize_locals_and_globals_mixin import (
    InitializeLocalsAndGlobalsMixIn,
)
from apysc._type.int import Int
from apysc._validation import arg_validation_decos


class ForArrayIndices(
    ForLoopExitMixIn,
    GetLastScopeInterface,
    InitializeLocalsAndGlobalsMixIn,
):
    """
    The loop implementation class for the `ap.Array` indices.

    References
    ----------
    - ForArrayIndices class
        - https://simon-ritchie.github.io/apysc/en/for_array_indices.html
    - Why the apysc library doesn’t use the Python built-in data type
        - https://simon-ritchie.github.io/apysc/en/why_apysc_doesnt_use_python_builtin_data_type.html  # noqa
    - Each branch instruction class’s scope variable reverting setting
        - https://simon-ritchie.github.io/apysc/en/branch_instruction_variables_reverting_setting.html  # noqa

    Examples
    --------
    >>> import apysc as ap

    >>> arr: ap.Array[ap.Number] = ap.Array(
    ...     [ap.Number(50), ap.Number(150), ap.Number(250)]
    ... )
    >>> indices: ap.Array[ap.Int] = ap.Array([])
    >>> with ap.ForArrayIndices(arr=arr) as i:
    ...     indices.append(i)
    ...

    >>> _ = ap.assert_arrays_equal(indices, [0, 1, 2])
    """

    _arr: Array
    _variable_name_suffix: str

    @final
    @arg_validation_decos.is_apysc_array(arg_position_index=1)
    @arg_validation_decos.is_builtin_string(arg_position_index=4, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        arr: Array,
        *,
        locals_: Optional[Dict[str, Any]] = None,
        globals_: Optional[Dict[str, Any]] = None,
        variable_name_suffix: str = "",
    ) -> None:
        """
        The loop implementation class for the `ap.Array` indices.

        Parameters
        ----------
        arr : Array
            An array to iterate.
        locals_ : Optional[Dict[str, Any]], optional
            Current scope's local variables. Set locals()
            value to this argument. If specified, this interface
            reverts all local scope VariableNameMixIn
            variables (like Int, Sprite) at the end of a with-statement
            scope. This setting is useful when you don't want to
            update each variable.
        globals_ : Optional[Dict[str, Any]], optional
            Current scope's global variables. Set globals()
            value to this argument. This setting works
            the same way as the locals_ argument.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        References
        ----------
        - ForArrayIndices class
            - https://simon-ritchie.github.io/apysc/en/for_array_indices.html
        - Why the apysc library doesn’t use the Python built-in data type
            - https://simon-ritchie.github.io/apysc/en/why_apysc_doesnt_use_python_builtin_data_type.html  # noqa
        - Each branch instruction class’s scope variable reverting setting
            - https://simon-ritchie.github.io/apysc/en/branch_instruction_variables_reverting_setting.html  # noqa

        Examples
        --------
        >>> import apysc as ap

        >>> arr: ap.Array[ap.Number] = ap.Array(
        ...     [ap.Number(50), ap.Number(150), ap.Number(250)]
        ... )
        >>> indices: ap.Array[ap.Int] = ap.Array([])
        >>> with ap.ForArrayIndices(arr=arr) as i:
        ...     indices.append(i)
        ...

        >>> _ = ap.assert_arrays_equal(indices, [0, 1, 2])
        """
        self._initialize_locals_and_globals(locals_=locals_, globals_=globals_)
        self._arr = arr
        self._variable_name_suffix = variable_name_suffix
        self._indent = Indent()

    @final
    @add_debug_info_setting(module_name=__name__)
    def __enter__(self) -> Int:
        """
        The entering method for the beginning of with-statement.

        Returns
        -------
        i : Int
            An index of iteration.
        """
        from apysc._loop import loop_count
        from apysc._type import revert_mixin

        loop_count.increment_current_loop_count()
        self._snapshot_name = revert_mixin.make_snapshots_of_each_scope_vars(
            locals_=self._locals, globals_=self._globals
        )
        i: Int = Int(0, variable_name_suffix=self._variable_name_suffix)
        self._append_enter_expression(i=i)
        self._indent.__enter__()
        return i

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_enter_expression(self, *, i: Int) -> None:
        """
        Append a for-loop start expression.

        Parameters
        ----------
        i : Int
            Loop index value.
        """
        from apysc._expression import expression_data_util

        arr_name: str = self._arr.variable_name
        i_name: str = i.variable_name
        expression: str = (
            f"for ({i_name} = 0; {i_name} < {arr_name}.length; {i_name}++) {{"
        )
        expression_data_util.append_js_expression(expression=expression)

    @final
    def _get_last_scope(self) -> LastScope:
        """
        Get a target last scope value.

        Returns
        -------
        last_scope : LastScope
            A target last scope.
        """
        return LastScope.FOR_ARRAY_INDICES



"""The mix-in class implementation for the `y_max` value.
"""

from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number
from apysc._validation import arg_validation_decos


class YMaxMixIn:
    _y_max: Optional[Number]

    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=True)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_y_max(
        self,
        *,
        y_max: Optional[Union[float, Number]],
        variable_name_suffix: str = "",
    ) -> None:
        """
        Set an initial maximum y-axis value.

        Parameters
        ----------
        y_max : Optional[Union[float, Number]]
            A maximum y-axis value.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        if y_max is None:
            self._y_max = None
            return
        if not isinstance(y_max, Number):
            y_max_: Number = Number(y_max, variable_name_suffix=variable_name_suffix)
        else:
            y_max_ = y_max
        self._y_max = y_max_



"""This module is for the translation mapping data of the
following document:

Document file: timer_delay.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Timer class delay setting": "# Timer クラスの delay 設定",
    ##################################################
    "This page explains the `Timer` class `delay` argument setting.": "このページでは`Timer`クラスの`delay`引数の設定について説明します。",  # noqa
    ##################################################
    "## What argument is this?": "## 引数の概要",
    ##################################################
    "The `delay` argument setting determines the timer tick interval. This setting is a milliseconds unit, so a value of 1000 ticks every 1 second.": "`delay`引数の設定ではタイマーの間隔を設定できます。この設定はミリ秒単位となり、1000の値を指定すれば1秒ごとの間隔になります。",  # noqa
    ##################################################
    "The `int`\\, `float`\\, `Int`\\, `Number`\\, and `FPS` enum can be acceptable.": "この引数は`int`、`float`、`Int`、`Number`型の値、もしくは`FPS`のenumの値を受け付けます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "You can set the `delay` parameter at the `Timer` class constructor. The following example sets each timer (`timer_1`, `timer_2`, `timer_3`) and passes the delay values of `100`, `333.3333333`, `16.6666667`.": "`Timer`クラスのコンストラクタで`deplay`引数のパラメーターを設定することができます。以下のコード例では`timer_1`、`timer_2`、`timer_3`の3つのタイマーを生成し、それぞれdelayの値に`100`、`33.3333333`、`16.6666667`の各値を設定しています。",  # noqa
    ##################################################
    "The first-timer (`delay` is 100) is called 10 times in a second, and the second one (`delay` is 33.3333333) is 30 times in a second, and the third one (`delay` is 16.6666667) is 60 times.": "1番目のタイマー（`delay`の値は100）では1秒間に10回ハンドラの呼び出しを行い、2番目のタイマー（`delay`の値は33.3333333）では1秒間に30回の呼び出しを行い、3番目のタイマー（`delay`の値は16.6666667）では60回の呼び出しを行います。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nap.Stage(\n    stage_width=350,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(x=250, y=50, width=50, height=50)\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The Handler would be called every timer tick.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.rotation_around_center += 1\n\n\noptions: _RectOptions = {"rectangle": rectangle_1}\ntimer_1: ap.Timer = ap.Timer(handler=on_timer, delay=100, options=options)\ntimer_1.start()\n\noptions = {"rectangle": rectangle_2}\ntimer_2: ap.Timer = ap.Timer(handler=on_timer, delay=33.3333333, options=options)\ntimer_2.start()\n\noptions = {"rectangle": rectangle_3}\ntimer_3: ap.Timer = ap.Timer(handler=on_timer, delay=16.6666667, options=options)\ntimer_3.start()\n\nap.save_overall_html(dest_dir_path="timer_delay_basic_usage/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nap.Stage(\n    stage_width=350,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(x=250, y=50, width=50, height=50)\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The Handler would be called every timer tick.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.rotation_around_center += 1\n\n\noptions: _RectOptions = {"rectangle": rectangle_1}\ntimer_1: ap.Timer = ap.Timer(handler=on_timer, delay=100, options=options)\ntimer_1.start()\n\noptions = {"rectangle": rectangle_2}\ntimer_2: ap.Timer = ap.Timer(handler=on_timer, delay=33.3333333, options=options)\ntimer_2.start()\n\noptions = {"rectangle": rectangle_3}\ntimer_3: ap.Timer = ap.Timer(handler=on_timer, delay=16.6666667, options=options)\ntimer_3.start()\n\nap.save_overall_html(dest_dir_path="timer_delay_basic_usage/")\n```',  # noqa
    ##################################################
    "## Set the FPS enum value to the delay argument": "## delay 引数にFPSのenumの値を設定する",
    ##################################################
    "You can also pass the `FPS` (frames per second) enum value to the `delay` argument. For example, if the `FPS.FPS_60` is specified, a timer delay becomes 60 frames per second (16.6666667 milliseconds). Likely, the `FPS.FPS_30` is specified, a timer delay becomes 30 frames per second (33.3333333 milliseconds).": "`delay`の引数には`FPS`（frames per second / 1秒当たりのフレーム数）のenumの値を指定することもできます。例えば、`FPS.FPS_60`を指定すれば60FPS相当の実行回数（16.6666667ミリ秒ごとの実行）となります。同様に`FPS.FPS_30`を指定すれば30FPS相当（33.3333333ミリ秒ごとの実行）となります。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The Handler would be called every timer tick.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.rotation_around_center += 1\n\n\noptions: _RectOptions = {"rectangle": rectangle}\ntimer: ap.Timer = ap.Timer(handler=on_timer, delay=ap.FPS.FPS_60, options=options)\ntimer.start()\n\nap.save_overall_html(dest_dir_path="timer_delay_fps_enum/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The Handler would be called every timer tick.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.rotation_around_center += 1\n\n\noptions: _RectOptions = {"rectangle": rectangle}\ntimer: ap.Timer = ap.Timer(handler=on_timer, delay=ap.FPS.FPS_60, options=options)\ntimer.start()\n\nap.save_overall_html(dest_dir_path="timer_delay_fps_enum/")\n```',  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [FPS enum](fps.md)": "- [FPS の enum](jp_fps.md)",
    ##################################################
    "## Timer constructor API": "## Timer クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Timer class to handle function calling at regular intervals.<hr>": "一定間隔ごとにハンドラの関数を実行するためのタイマーのクラスです。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `handler`: _Handler": "- `handler`: _Handler",
    ##################################################
    "  - A handler would be called at regular intervals.": "  - 一定間隔ごとに呼ばれる関数もしくはメソッドのハンドラ。",  # noqa
    ##################################################
    "- `delay`: Int or int or Number or float or FPS": "- `delay`: Int or int or Number or float or FPS",  # noqa
    ##################################################
    "  - A delay between each `Handler` calling in a millisecond or FPS value. If an `FPS` value is specified, this value becomes a millisecond calculated with that FPS value (e.g., if the `FPS_60` value is specified, then `delay` becomes 16.6666667).": "  - ハンドラの実行間隔となるミリ秒もしくはFPSのenumの値。もし`FPS`の値が指定された場合、FPSに応じて計算されたミリ秒が設定されます（例えば、もし`FPS_60`が指定されていれば`delay`の値は16.6666667ミリ秒相当になります。）。",  # noqa
    ##################################################
    "- `repeat_count`: Int or int": "- `repeat_count`: Int or int",
    ##################################################
    "  - Max count of a `Handler`'s calling. A timer stops if the `Handler`'s calling count has reached this value. If 0 is specified, then a timer loops forever.": "  - ハンドラの実行回数の上限値。ハンドラの実行回数がこの値に到達した場合タイマーは停止します。もし0が指定された場合にはタイマーは停止しなくなります。",  # noqa
    ##################################################
    "- `options`: dict or None, default None": "- `options`: dict or None, default None",  # noqa
    ##################################################
    "  - Optional arguments dictionary to pass a `Handler` callable.": "  - ハンドラの関数もしくはメソッドへ渡すオプションとしての各パラメーターを格納した辞書。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.x += 1\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> _ = ap.Timer(on_timer, delay=ap.FPS.FPS_60, options=options).start()\n```': '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.x += 1\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> _ = ap.Timer(on_timer, delay=ap.FPS.FPS_60, options=options).start()\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [Timer](https://simon-ritchie.github.io/apysc/en/timer.html)": "- [Timer クラス](https://simon-ritchie.github.io/apysc/jp/jp_timer.html)",  # noqa
    ##################################################
    "- [TimerEvent class](https://simon-ritchie.github.io/apysc/en/timer_event.html)": "- [TimerEvent クラス](https://simon-ritchie.github.io/apysc/jp/jp_timer_event.html)",  # noqa
    ##################################################
    "- [FPS enum](https://simon-ritchie.github.io/apysc/en/fps.html)": "- [FPS の enum](https://simon-ritchie.github.io/apysc/jp/jp_fps.html)",  # noqa
    ##################################################
    "- [Timer class repeat_count setting](https://simon-ritchie.github.io/apysc/en/timer_repeat_count.html)": "- [Timer クラスの repeat_count 設定](https://simon-ritchie.github.io/apysc/jp/jp_timer_repeat_count.html)",  # noqa
    ##################################################
    "- [About the handler options' type](https://simon-ritchie.github.io/apysc/en/about_handler_options_type.html)": "- [ハンドラのoptions引数の型について](https://simon-ritchie.github.io/apysc/jp/jp_about_handler_options_type.html)",  # noqa
    ##################################################
    "## delay property API": "## delay 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a delay value.<hr>": "遅延（間隔）値を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `delay`: Number": "- `delay`: Number",
    ##################################################
    "  - A delay value of each `Handler` calling in milliseconds.": "  - ハンドラの実行ごとのミリ秒の間隔値。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> def on_timer(e: ap.TimerEvent, options: dict) -> None:\n...     pass\n>>> timer: ap.Timer = ap.Timer(on_timer, delay=33.3, repeat_count=50)\n>>> timer.delay\nNumber(33.3)\n```": "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> def on_timer(e: ap.TimerEvent, options: dict) -> None:\n...     pass\n>>> timer: ap.Timer = ap.Timer(on_timer, delay=33.3, repeat_count=50)\n>>> timer.delay\nNumber(33.3)\n```",  # noqa
}



"""The class implementation for the `mic_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialMicOffIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `mic_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0zm0 0h24v24H0z" fill="none"/><path d="M19 11h-1.7c0 .74-.16 1.43-.43 2.05l1.23 1.23c.56-.98.9-2.09.9-3.28zm-4.02.17c0-.06.02-.11.02-.17V5c0-1.66-1.34-3-3-3S9 3.34 9 5v.18l5.98 5.99zM4.27 3L3 4.27l6.01 6.01V11c0 1.66 1.33 3 2.99 3 .22 0 .44-.03.65-.08l1.66 1.66c-.71.33-1.5.52-2.31.52-2.76 0-5.3-2.1-5.3-5.1H5c0 3.41 2.72 6.23 6 6.72V21h2v-3.28c.91-.13 1.77-.45 2.54-.9L19.73 21 21 19.73 4.27 3z"/></svg>'  # noqa



"""The class implementation for the outlined `calculate` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialCalculateOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `calculate` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><path d="M19,3H5C3.9,3,3,3.9,3,5v14c0,1.1,0.9,2,2,2h14c1.1,0,2-0.9,2-2V5C21,3.9,20.1,3,19,3z M19,19H5V5h14V19z"/><rect height="1.5" width="5" x="6.25" y="7.72"/><rect height="1.5" width="5" x="13" y="15.75"/><rect height="1.5" width="5" x="13" y="13.25"/><polygon points="8,18 9.5,18 9.5,16 11.5,16 11.5,14.5 9.5,14.5 9.5,12.5 8,12.5 8,14.5 6,14.5 6,16 8,16"/><polygon points="14.09,10.95 15.5,9.54 16.91,10.95 17.97,9.89 16.56,8.47 17.97,7.06 16.91,6 15.5,7.41 14.09,6 13.03,7.06 14.44,8.47 13.03,9.89"/></g></g></svg>'  # noqa



"""The class implementation for the outlined `mic_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialMicOffOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `mic_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M10.8 4.9c0-.66.54-1.2 1.2-1.2s1.2.54 1.2 1.2l-.01 3.91L15 10.6V5c0-1.66-1.34-3-3-3-1.54 0-2.79 1.16-2.96 2.65l1.76 1.76V4.9zM19 11h-1.7c0 .58-.1 1.13-.27 1.64l1.27 1.27c.44-.88.7-1.87.7-2.91zM4.41 2.86L3 4.27l6 6V11c0 1.66 1.34 3 3 3 .23 0 .44-.03.65-.08l1.66 1.66c-.71.33-1.5.52-2.31.52-2.76 0-5.3-2.1-5.3-5.1H5c0 3.41 2.72 6.23 6 6.72V21h2v-3.28c.91-.13 1.77-.45 2.55-.9l4.2 4.2 1.41-1.41L4.41 2.86z"/></svg>'  # noqa



"""The class implementation for the outlined `http` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialHttpOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `http` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M24 24H0V0h24v24z" fill="none" opacity=".87"/><path d="M4.5 11h-2V9H1v6h1.5v-2.5h2V15H6V9H4.5v2zm2.5-.5h1.5V15H10v-4.5h1.5V9H7v1.5zm5.5 0H14V15h1.5v-4.5H17V9h-4.5v1.5zm9-1.5H18v6h1.5v-2h2c.8 0 1.5-.7 1.5-1.5v-1c0-.8-.7-1.5-1.5-1.5zm0 2.5h-2v-1h2v1z"/></svg>'  # noqa



"""Class implementation for the enter frame mix-in.
"""

from typing import Callable
from typing import Dict
from typing import Generic
from typing import Optional
from typing import TypeVar

from typing_extensions import final

from apysc._event.enter_frame_event import EnterFrameEvent
from apysc._event.handler import HandlerData
from apysc._event.set_handler_data_mixin import SetHandlerDataMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._time.datetime_ import DateTime
from apysc._time.fps import FPS
from apysc._time.fps import FPSDefinition
from apysc._type.boolean import Boolean
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)
from apysc._validation import arg_validation_decos

_Options = TypeVar("_Options")
_HandlerName = str
_Target = TypeVar("_Target")


class _EnterFrameEventNotRegistered(Exception):
    pass


class EnterFrameMixIn(
    VariableNameSuffixAttrOrVarMixIn,
    SetHandlerDataMixIn[EnterFrameEvent],
    Generic[_Target, _Options],
    VariableNameMixIn,
):
    _enter_frame_handlers: Dict[str, HandlerData[EnterFrameEvent]]
    _is_stopped_settings: Dict[_HandlerName, Boolean]
    _fps_millisecond_intervals_settings: Dict[_HandlerName, Number]
    _loop_func_name_settings: Dict[_HandlerName, str]
    _prev_time_settings: Dict[_HandlerName, DateTime]

    @final
    @arg_validation_decos.handler_args_num(arg_position_index=1)
    @arg_validation_decos.is_fps(arg_position_index=2)
    @arg_validation_decos.handler_options_type(arg_position_index=3)
    @add_debug_info_setting(module_name=__name__)
    def enter_frame(
        self,
        handler: Callable[[EnterFrameEvent, _Options], None],
        *,
        fps: FPS = FPS.FPS_60,
        options: Optional[_Options] = None,
    ) -> None:
        """
        Add an enter frame event listener setting.

        Notes
        -----
        If this is the second call of this interface and an argument is
        the same function, this interface ignores `options`
        argument (it changes only the running status and `fps` setting).

        Parameters
        ----------
        handler : Callable[[EnterFrameEvent, _Options], None]
            A handler function to handle the enter frame event.
        fps : FPS, default FPS.FPS_60
            Frame per second to set.
        options : Optional[_Options], optional
            Optional arguments to pass to a handler function.

        References
        ----------
        - enter_frame interface
            - https://simon-ritchie.github.io/apysc/en/enter_frame.html

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> rectangle: ap.Rectangle = ap.Rectangle(
        ...     x=50, y=50, width=50, height=50, fill_color=ap.Color("#0af")
        ... )
        >>> def on_enter_frame(e: ap.EnterFrameEvent, options: dict) -> None:
        ...     rectangle.x += 1
        >>> stage.enter_frame(handler=on_enter_frame, fps=ap.FPS.FPS_30)
        """
        from apysc._event.enter_frame_event import EnterFrameEvent
        from apysc._event.handler import append_handler_expression
        from apysc._event.handler import get_handler_name
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names

        self._initialize_enter_frame_handlers_if_not_initialized()
        self._initialize_is_stopped_settings_if_not_initialized()
        self._initialize_fps_millisecond_intervals_settings_if_not_initialized()
        self._initialize_loop_func_name_settings_if_not_initialized()
        self._initialize_prev_time_settings_if_not_initialized()

        handler_name: str = get_handler_name(handler=handler, instance=self)
        if handler_name in self._is_stopped_settings:
            self._append_enter_frame_rebinding_expression(
                handler_name=handler_name,
                fps=fps,
            )
            return

        LOOP_FUNC_NAME: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.LOOP
        )
        self._set_handler_data(
            handler=handler,
            handlers_dict=self._enter_frame_handlers,
            options=options,
        )
        is_stopped: Boolean = Boolean(
            False,
            variable_name_suffix=self._get_attr_or_variable_name_suffix(
                value_identifier="is_stopped",
            ),
        )
        prev_time: DateTime = DateTime.now(
            variable_name_suffix=self._get_attr_or_variable_name_suffix(
                value_identifier="prev_time",
            ),
        )
        event: EnterFrameEvent = EnterFrameEvent(this=self)
        millisecond_interval: Number = _get_millisecond_interval_from_fps(fps=fps)
        self._append_enter_frame_expression(
            handler_name=handler_name,
            millisecond_interval=millisecond_interval,
            is_stopped=is_stopped,
            loop_func_name=LOOP_FUNC_NAME,
            prev_time=prev_time,
        )
        append_handler_expression(
            handler_data=self._enter_frame_handlers[handler_name],
            handler_name=handler_name,
            e=event,
        )
        self._is_stopped_settings[handler_name] = is_stopped
        self._loop_func_name_settings[handler_name] = LOOP_FUNC_NAME
        self._prev_time_settings[handler_name] = prev_time
        self._fps_millisecond_intervals_settings[handler_name] = millisecond_interval

    def _append_enter_frame_rebinding_expression(
        self,
        *,
        handler_name: str,
        fps: FPS,
    ) -> None:
        """
        Append an enter-frame's rebinding expression string.

        Parameters
        ----------
        handler_name : str
            Target handler's name.
        fps : FPS, default FPS.FPS_60
            Frame per second to set.
        """
        from apysc._branch._if import If
        from apysc._expression import expression_data_util
        from apysc._time.datetime_ import DateTime

        with If(self._is_stopped_settings[handler_name]):
            self._is_stopped_settings[handler_name].value = False
            self._fps_millisecond_intervals_settings[handler_name].value = (
                _get_millisecond_interval_from_fps(fps=fps)
            )
            prev_time: DateTime = self._prev_time_settings[handler_name]
            now: DateTime = DateTime.now()
            prev_time.year = now.year
            prev_time.month = now.month
            prev_time.day = now.day
            prev_time.hour = now.hour
            prev_time.minute = now.minute
            prev_time.second = now.second
            prev_time.millisecond = now.millisecond
            expression_data_util.append_js_expression(
                expression=(
                    "requestAnimationFrame("
                    f"{self._loop_func_name_settings[handler_name]});"
                ),
            )

    def _initialize_prev_time_settings_if_not_initialized(self) -> None:
        """
        Initialize the `_prev_time_settings`'s attribute
        if this instance has not initialized it yet.
        """
        if hasattr(self, "_prev_time_settings"):
            return
        self._prev_time_settings = {}

    def _initialize_loop_func_name_settings_if_not_initialized(self) -> None:
        """
        Initialize the `_loop_func_name_settings`'s attribute
        if this instance has not initialized it yet.
        """
        if hasattr(self, "_loop_func_name_settings"):
            return
        self._loop_func_name_settings = {}

    def _initialize_fps_millisecond_intervals_settings_if_not_initialized(self) -> None:
        """
        Initialize the `_fps_millisecond_intervals_settings`'s
        attribute if this instance has not initialized it yet.
        """
        if hasattr(self, "_fps_millisecond_intervals_settings"):
            return
        self._fps_millisecond_intervals_settings = {}

    def _append_enter_frame_expression(
        self,
        *,
        handler_name: str,
        millisecond_interval: Number,
        is_stopped: Boolean,
        loop_func_name: str,
        prev_time: DateTime,
    ) -> None:
        """
        Append an enter frame expression string.

        Parameters
        ----------
        handler_name : str
            Target handler's name.
        millisecond_interval : Number
            Millisecond interval. This value depends on an FPS setting.
        is_stopped : Boolean
            A boolean to control an animation loop.
        loop_func_name : str
            A loop function name to set.
        prev_time : DateTime
            Previous time to calculate the duration.
        """
        from apysc._expression import expression_data_util
        from apysc._expression.indent_num import Indent
        from apysc._math.math import Math
        from apysc._time.timedelta_ import TimeDelta
        from apysc._type.array import Array

        expression: str = (
            f"function {loop_func_name}() {{"
            f"\n  if ({is_stopped.variable_name}) {{"
            "\n    return;"
            "\n  }"
        )
        expression_data_util.append_js_expression(expression=expression)
        with Indent():
            current_time: DateTime = DateTime.now(
                variable_name_suffix=self._get_attr_or_variable_name_suffix(
                    value_identifier="current_time",
                )
            )
            count: Int = Int(
                0,
                variable_name_suffix=self._get_attr_or_variable_name_suffix(
                    value_identifier="count",
                ),
            )
            liimted_count: Int = Int(
                0,
                variable_name_suffix=self._get_attr_or_variable_name_suffix(
                    value_identifier="limited_count",
                ),
            )
            timedelta_: TimeDelta = current_time - prev_time
            total_milliseconds: Number = timedelta_.total_seconds() * 1000
            limited_total_milliseconds: Number = Math.min(
                Array([total_milliseconds, 1000])
            )
            count.value = Math.trunc(total_milliseconds / millisecond_interval)
            liimted_count.value = Math.trunc(
                limited_total_milliseconds / millisecond_interval
            )
            expression_data_util.append_js_expression(
                expression=(
                    f"for (var i = 0; i < {liimted_count.variable_name}; i++) {{"
                    f"\n  {handler_name}();"
                    "\n}"
                )
            )
            expression_data_util.append_js_expression(
                expression=f"requestAnimationFrame({loop_func_name});"
            )
            prev_time.millisecond += count * millisecond_interval
        expression_data_util.append_js_expression(expression="}")
        expression_data_util.append_js_expression(
            expression=f"requestAnimationFrame({loop_func_name});"
        )

    def _initialize_enter_frame_handlers_if_not_initialized(self) -> None:
        """
        Initialize the `_enter_frame_handlers`'s attribute if this instance
        has not initialized it yet.
        """
        if hasattr(self, "_enter_frame_handlers"):
            return
        self._enter_frame_handlers = {}

    def _initialize_is_stopped_settings_if_not_initialized(self) -> None:
        """
        Initialize the `_is_stopped_settings`'s attribute if this instance
        has not initialized it yet.
        """
        if hasattr(self, "_is_stopped_settings"):
            return
        self._is_stopped_settings = {}

    @final
    @arg_validation_decos.handler_args_num(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def unbind_enter_frame(
        self,
        handler: Callable[[EnterFrameEvent, _Options], None],
    ) -> None:
        """
        Unbind a specified handler's enter-frame event.

        Parameters
        ----------
        handler : Callable[[EnterFrameEvent, _Options], None]
            Unbinding target callable.

        Raises
        ------
        _EnterFrameEventNotRegistered
            If there is no unbinding target of a specified handler.

        References
        ----------
        - unbind_enter_frame and unbind_enter_frame_all interfaces
            - https://simon-ritchie.github.io/apysc/en/unbind_enter_frame_and_unbind_enter_frame_all.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> rectangle: ap.Rectangle = ap.Rectangle(
        ...     x=50, y=50, width=50, height=50, fill_color=ap.Color("#0af")
        ... )
        >>> def on_enter_frame(e: ap.EnterFrameEvent, options: dict) -> None:
        ...     rectangle.x += 1
        >>> stage.enter_frame(handler=on_enter_frame, fps=ap.FPS.FPS_30)
        >>> # Any implementations here...
        >>> stage.unbind_enter_frame(handler=on_enter_frame)
        """
        from apysc._event.handler import get_handler_name

        self._initialize_is_stopped_settings_if_not_initialized()
        handler_name: str = get_handler_name(handler=handler, instance=self)
        if handler_name not in self._is_stopped_settings:
            raise _EnterFrameEventNotRegistered(
                "There is no unbinding target of a specified handler."
                "\nPlease call `enter_frame` interframe before call this method."
            )
        self._is_stopped_settings[handler_name].value = True

    @final
    @add_debug_info_setting(module_name=__name__)
    def unbind_enter_frame_all(self) -> None:
        """
        Unbind all enter-frame events.

        References
        ----------
        - unbind_enter_frame and unbind_enter_frame_all interfaces
            - https://simon-ritchie.github.io/apysc/en/unbind_enter_frame_and_unbind_enter_frame_all.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> rectangle: ap.Rectangle = ap.Rectangle(
        ...     x=50, y=50, width=50, height=50, fill_color=ap.Color("#0af")
        ... )
        >>> def on_enter_frame(e: ap.EnterFrameEvent, options: dict) -> None:
        ...     rectangle.x += 1
        >>> stage.enter_frame(handler=on_enter_frame, fps=ap.FPS.FPS_30)
        >>> # Any implementations here...
        >>> stage.unbind_enter_frame_all()
        """
        self._initialize_is_stopped_settings_if_not_initialized()
        for is_stopped in self._is_stopped_settings.values():
            is_stopped.value = True


def _get_millisecond_interval_from_fps(*, fps: FPS) -> Number:
    """
    Get a millisecond interval value from a specified FPS.

    Parameters
    ----------
    fps : FPS
        Frame per second.

    Returns
    -------
    millisecond_interval : Number
        A created millisecond interval value.
    """
    fps_val: FPSDefinition = fps.value
    millisecond_interval: Number = Number(fps_val.millisecond_interval)
    return millisecond_interval



"""The class implementation for the material design filled button.
"""

from typing import List
from typing import Optional
from typing import Union

from apysc._color.color import Color
from apysc._display.add_to_parent_mixin import AddToParentMixIn
from apysc._display.child_mixin import ChildMixIn
from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase
from apysc._display.sprite import Sprite
from apysc._geom.rectangle_geom import RectangleGeom
from apysc._material_design.button.material_button_const import BUTTON_HEIGHT
from apysc._material_design.button.material_button_const import ICON_SIZE
from apysc._material_design.button.material_button_const import NO_ICON_OUTER_PADDING
from apysc._material_design.button.material_button_icon_mixin import (
    MaterialButtonIconMixIn,
)
from apysc._material_design.button.material_button_label_mixin import (
    MaterialButtonLabelMixIn,
)
from apysc._material_design.geom.material_x_and_y_attributes_mixin import (
    MaterialXAndYAttributesMixIn,
)
from apysc._type.array import Array
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String
from apysc._validation import arg_validation_decos


class MaterialFilledButton(
    Sprite,
    MaterialButtonLabelMixIn,
    MaterialButtonIconMixIn,
    MaterialXAndYAttributesMixIn,
    AddToParentMixIn,
):
    """
    The class for the material design filled button.

    References
    ----------
    - Material design filled button document
        - https://m3.material.io/components/buttons/specs#0b1b7bd2-3de8-431a-afa1-d692e2e18b0d  # noqa
    """

    _background_color: Color

    # label
    @arg_validation_decos.is_string(arg_position_index=1, optional=False)
    # prefix_icon
    @arg_validation_decos.is_fixed_html_svg_icon(arg_position_index=2, optional=True)
    # suffix_icon
    @arg_validation_decos.is_fixed_html_svg_icon(arg_position_index=3, optional=True)
    # x
    @arg_validation_decos.is_num(arg_position_index=4, optional=False)
    # y
    @arg_validation_decos.is_num(arg_position_index=5, optional=False)
    # text_color
    @arg_validation_decos.is_color(arg_position_index=6, optional=True)
    # font_family
    @arg_validation_decos.is_builtin_str_list_or_apysc_str_arr(
        arg_position_index=7, optional=True
    )
    # font_size
    @arg_validation_decos.is_integer(arg_position_index=8, optional=False)
    # background_color
    @arg_validation_decos.is_color(arg_position_index=9, optional=True)
    # parent
    @arg_validation_decos.is_display_object_container(
        arg_position_index=10, optional=True
    )
    # variable_name_suffix
    @arg_validation_decos.is_builtin_string(arg_position_index=11, optional=False)
    def __init__(
        self,
        *,
        label: Union[str, String],
        prefix_icon: Optional[FixedHtmlSvgIconBase] = None,
        suffix_icon: Optional[FixedHtmlSvgIconBase] = None,
        x: Union[float, Number] = 0.0,
        y: Union[float, Number] = 0.0,
        text_color: Optional[Color] = None,
        font_family: Optional[Union[Array[String], List[str]]] = None,
        font_size: Union[int, Int] = 14,
        background_color: Optional[Color] = None,
        parent: Optional[ChildMixIn] = None,
        variable_name_suffix: str = "",
    ) -> None:
        """
        The class for the material design filled button.

        References
        ----------
        - Material design filled button document
            - https://m3.material.io/components/buttons/specs#0b1b7bd2-3de8-431a-afa1-d692e2e18b0d  # noqa

        Parameters
        ----------
        label : Union[str, String]
            The label text to display on this button.
        prefix_icon : Optional[FixedHtmlSvgIconBase], optional
            An icon to display on the left side of the label.
        suffix_icon : Optional[FixedHtmlSvgIconBase], optional
            An icon to display on the right side of the label.
        x : Union[float, Number], optional
            X-coordinate of this button.
        y : Union[float, Number], optional
            Y-coordinate of this button.
        text_color : Optional[Color], optional
            The color of the label text.
            The label color becomes according to the following priorities:
            1. If this argument is not omitted (i.e., it is not `None`)
            2. If a color scheme is set in the `MaterialSettings` (this button
                refers to the `primary` color)
            3. A fixed color value.
        font_family : Optional[Union[Array[String], List[str]]], optional
            A font-family setting.
        font_size : Union[int, Int], optional
            A font-size setting.
        background_color : Optional[Color], optional
            The background color of this button.
            The background color becomes according to the following priorities:
            1. If this argument is not omitted (i.e., it is not `None`)
            2. If a color scheme is set in the `MaterialSettings` (this button
                refers to the `on_primary` color)
            3. A fixed color value.
        parent : Optional[ChildMixIn], optional
            A parent instance to add this instance.
            If the specified value is None, this interface uses
            a stage instance.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        from apysc._material_design.setting.material_settings_utils import (
            MaterialSettingsUtils,
        )

        self._variable_name_suffix = variable_name_suffix
        variable_name: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.MATERIAL_FILLED_BUTTON,
        )
        super(MaterialFilledButton, self).__init__(
            variable_name=variable_name, variable_name_suffix=variable_name_suffix
        )

        on_primary_color: Color = MaterialSettingsUtils.get_on_primary_color(
            argument_color=text_color
        )
        self._initialize_label(
            label=label,
            text_color=on_primary_color,
            font_family=font_family,
            font_size=font_size,
        )
        self._background_color = MaterialSettingsUtils.get_primary_color(
            argument_color=background_color
        )
        label_text_initial_bounding_box: RectangleGeom = self._label_text.get_bounds(
            target_coordinate_space_object=self
        )
        self._resize_icon_size(prefix_icon=prefix_icon, suffix_icon=suffix_icon)
        self._add_icons(prefix_icon=prefix_icon, suffix_icon=suffix_icon)
        self._redraw_background(
            label_text_bounding_box=label_text_initial_bounding_box,
            prefix_icon=prefix_icon,
            suffix_icon=suffix_icon,
        )
        background_initial_bounding_box: RectangleGeom = self.get_bounds()
        self._locate_icons(
            prefix_icon=prefix_icon,
            suffix_icon=suffix_icon,
            background_bounding_box=background_initial_bounding_box,
        )
        self._locate_label_text(
            label_text_bounding_box=label_text_initial_bounding_box,
            prefix_icon=prefix_icon,
        )
        self._set_fill_color_to_icons(
            color=on_primary_color, prefix_icon=prefix_icon, suffix_icon=suffix_icon
        )
        self._set_x_and_y_coordinates(x=x, y=y)
        self._add_to_parent(parent=parent)

    def _redraw_background(
        self,
        *,
        label_text_bounding_box: RectangleGeom,
        prefix_icon: Optional[FixedHtmlSvgIconBase],
        suffix_icon: Optional[FixedHtmlSvgIconBase],
    ) -> None:
        """
        Redraw the background of this button.

        Parameters
        ----------
        label_text_bounding_box : RectangleGeom
            The bounding box of the label text.
        prefix_icon : Optional[FixedHtmlSvgIconBase]
            An icon to display on the left side of the label.
        suffix_icon : Optional[FixedHtmlSvgIconBase]
            An icon to display on the right side of the label.
        """
        self.graphics.clear()
        self.graphics.begin_fill(color=self._background_color)
        additional_width: int = 0
        if prefix_icon is not None:
            additional_width += ICON_SIZE
        if suffix_icon is not None:
            additional_width += ICON_SIZE
        self.graphics.draw_round_rect(
            x=0,
            y=0,
            width=(
                label_text_bounding_box.width
                + NO_ICON_OUTER_PADDING * 2
                + additional_width
            ),
            height=BUTTON_HEIGHT,
            ellipse_width=int(BUTTON_HEIGHT / 2),
            ellipse_height=int(BUTTON_HEIGHT / 2),
        )



"""This module is for the translation mapping data of the
following document:

Document file: enter_frame.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# enter_frame interface": "# enter_frame インターフェイス",
    ##################################################
    "This page explains the `enter_frame` method interface.": "このページでは`enter_frame`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `enter_frame` interface sets a handler for an animation.": "`enter_frame`インターフェイスはアニメーションのためのハンドラを設定します。",  # noqa
    ##################################################
    "This interface calls a specified handler at each frame.": "このインターフェイスでは指定されたハンドラを各フレームごとに呼び出します。",  # noqa
    ##################################################
    "## Which should we use, the Timer class or the enter_frame interface?": "## Timer クラスと enter_frame のインターフェイスのどちらを使うべきか",  # noqa
    ##################################################
    "The `Timer` class also can handle animation.": "`Timer`クラスでも同様にアニメーションを扱うことができます。",  # noqa
    ##################################################
    "So, Which should we use, the `Timer` class or the `enter_frame` interface for an animation?": "そのため、アニメーション用途の場合`Timer`クラスと`enter_frame`のインターフェイスのどちらを使うべきなのでしょうか？",  # noqa
    ##################################################
    "The answer is, basically, the `enter_frame`.": "回答としては基本的には`enter_frame`側となります。",
    ##################################################
    "The `enter_frame` interface is less likely to shift the pace of handler calling.": "`enter_frame`のインターフェイスはハンドラの呼び出しの間隔が`Timer`クラスよりもずれにくくなっています。",  # noqa
    ##################################################
    "On the other hand, the `Timer` class's calling timing can be off if a CPU is busy.": "一方で、`Timer`クラス側はCPU負荷が高い場合などに呼び出しタイミングがずれるケースが発生します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `enter_frame` interface exists in classes, such as the `Stage` or `Sprite`.": "`enter_frame`インターフェイスは`Stage`や`Sprite`などのクラスに存在します。",  # noqa
    ##################################################
    "The `enter_frame` interface requires the `handler` argument (callable object, such as the function or method).": "`enter_frame`インターフェイスは`handler`引数（関数やメソッドなどのcallableオブジェクト）の指定を必要とします。",  # noqa
    ##################################################
    "The `fps` argument is optional and determines an animation's frame rate (it accepts the `FPS` enum).": "`fps`引数は省略可で、この引数はフレームレートを決定します（`FPS`のenumの指定が可能です）。",  # noqa
    ##################################################
    "Also, the `options` argument is an optional dictionary and passes optional parameters to a handler.": "また、`options`引数も省略可であり、この指定はハンドラへと追加のパラメーターを渡すことができます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Color("#0af"),\n)\n\n\ndef on_enter_frame(e: ap.EnterFrameEvent, options: dict) -> None:\n    """\n    The handler to handle an enter frame event.\n\n    Parameters\n    ----------\n    e : ap.EnterFrameEvent\n        Event instance.\n    options : dict\n        Optional argument dictionary.\n    """\n    rectangle.rotation_around_center += 1\n\n\nstage.enter_frame(handler=on_enter_frame, fps=ap.FPS.FPS_30)\nap.save_overall_html(dest_dir_path="enter_frame_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Color("#0af"),\n)\n\n\ndef on_enter_frame(e: ap.EnterFrameEvent, options: dict) -> None:\n    """\n    The handler to handle an enter frame event.\n\n    Parameters\n    ----------\n    e : ap.EnterFrameEvent\n        Event instance.\n    options : dict\n        Optional argument dictionary.\n    """\n    rectangle.rotation_around_center += 1\n\n\nstage.enter_frame(handler=on_enter_frame, fps=ap.FPS.FPS_30)\nap.save_overall_html(dest_dir_path="enter_frame_basic_usage/")\n```',  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [Timer class](timer.md)": "- [Timer クラス](jp_timer.md)",
    ##################################################
    "- [FPS enum](fps.md)": "- [FPS の enum](jp_fps.md)",
    ##################################################
    "## enter_frame API": "## enter_frame のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Add an enter frame event listener setting.<hr>": "enter frameのイベントのリスナー設定を追加します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `handler`: Callable[[EnterFrameEvent, _Options], None]": "- `handler`: Callable[[EnterFrameEvent, _Options], None]",  # noqa
    ##################################################
    "  - A handler function to handle the enter frame event.": "  - enter frameイベントを扱うためのハンドラの関数。",  # noqa
    ##################################################
    "- `fps`: FPS, default FPS.FPS_60": "- `fps`: FPS, default FPS.FPS_60",
    ##################################################
    "  - Frame per second to set.": "  - 設定する1秒辺りのフレーム数（frame per second）。",
    ##################################################
    "- `options`: Optional[_Options], optional": "- `options`: Optional[_Options], optional",  # noqa
    ##################################################
    "  - Optional arguments to pass to a handler function.": "  - ハンドラの関数へと渡される追加のパラメーターの引数値。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    "If this is the second call of this interface and an argument is the same function, this interface ignores `options` argument (it changes only the running status and `fps` setting).<hr>": "もしこのインターフェイスの呼び出しが2回目且つ指定されたハンドラの引数の値が同一の場合、このインターフェイスは`options`引数の指定を無視します（実行中かどうかのステータスと`fps`の設定のみ更新します）。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> rectangle: ap.Rectangle = ap.Rectangle(\n...     x=50, y=50, width=50, height=50, fill_color=ap.Color("#0af")\n... )\n>>> def on_enter_frame(e: ap.EnterFrameEvent, options: dict) -> None:\n...     rectangle.x += 1\n>>> stage.enter_frame(handler=on_enter_frame, fps=ap.FPS.FPS_30)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> rectangle: ap.Rectangle = ap.Rectangle(\n...     x=50, y=50, width=50, height=50, fill_color=ap.Color("#0af")\n... )\n>>> def on_enter_frame(e: ap.EnterFrameEvent, options: dict) -> None:\n...     rectangle.x += 1\n>>> stage.enter_frame(handler=on_enter_frame, fps=ap.FPS.FPS_30)\n```',  # noqa
}



"""The class implementation for the `translate` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialTranslateIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `translate` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M12.87 15.07l-2.54-2.51.03-.03c1.74-1.94 2.98-4.17 3.71-6.53H17V4h-7V2H8v2H1v1.99h11.17C11.5 7.92 10.44 9.75 9 11.35 8.07 10.32 7.3 9.19 6.69 8h-2c.73 1.63 1.73 3.17 2.98 4.56l-5.09 5.02L4 19l5-5 3.11 3.11.76-2.04zM18.5 10h-2L12 22h2l1.12-3h4.75L21 22h2l-4.5-12zm-2.62 7l1.62-4.33L19.12 17h-3.24z"/></svg>'  # noqa



"""The class implementation for the `call_split` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialCallSplitIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `call_split` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M14 4l2.29 2.29-2.88 2.88 1.42 1.42 2.88-2.88L20 10V4zm-4 0H4v6l2.29-2.29 4.71 4.7V20h2v-8.41l-5.29-5.3z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: path_close.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# PathClose class": "# PathClose クラス",
    ##################################################
    "This page explains the `PathClose` class.": "このページでは`PathClose`クラスについて説明します。",
    ##################################################
    "## What class is this?": "## クラス概要",
    ##################################################
    "The `PathClose` class is the class to close a path.": "`PathVertical`クラスはパス上に新しい垂直の直線の設定を追加するためのクラスです。",  # noqa
    ##################################################
    "If a path's start and end points are not connecting, this setting connects these points.": "もしもパスの始点と終点が繋がっていない場合、この設定はこれらの座標を接続します。",  # noqa
    ##################################################
    "Mainly, the `Path` class constructor or `draw_path` interfaces use this setting.": "主にこの設定は`Path`クラスのコンストラクタと`draw_path`メソッドのインターフェイスで使用されます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `PathClose` class constructor takes no arguments.": "`PathClose`クラスのコンストラクタは引数を必要としません。",  # noqa
    ##################################################
    "The `Path` class constructor or `draw_path` interfaces' `path_data_list` argument requires its instance.": "`Path`クラスのコンストラクタもしくは`draw_path`メソッドのインターフェイスの`path_data_list`引数でそのインスタンスが必要とされます。",  # noqa
    ##################################################
    "In the following example, the left path graphics do not use this `Close` class setting.": "以下のコード例では、左側のパスのグラフィックスは`Close`クラスの設定を使用していません。",  # noqa
    ##################################################
    "Conversely, the right path graphics use the `Close` class setting, and it connects the start and end points:": "逆に右側のパスのグラフィックスでは`Close`クラスの設定を使用しており、始点と終点の座標が接続されています:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=250,\n    stage_height=150,\n    stage_elem_id="stage",\n)\nleft_path: ap.Path = ap.Path(\n    path_data_list=[\n        ap.PathMoveTo(x=75, y=50),\n        ap.PathLineTo(x=50, y=100),\n        ap.PathLineTo(x=100, y=100),\n    ],\n    line_color=ap.Color("#0af"),\n    line_thickness=5,\n)\n\nright_path: ap.Path = ap.Path(\n    path_data_list=[\n        ap.PathMoveTo(x=175, y=50),\n        ap.PathLineTo(x=150, y=100),\n        ap.PathLineTo(x=200, y=100),\n        ap.PathClose(),\n    ],\n    line_color=ap.Color("#0af"),\n    line_thickness=5,\n)\n\nap.save_overall_html(dest_dir_path="path_close_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=250,\n    stage_height=150,\n    stage_elem_id="stage",\n)\nleft_path: ap.Path = ap.Path(\n    path_data_list=[\n        ap.PathMoveTo(x=75, y=50),\n        ap.PathLineTo(x=50, y=100),\n        ap.PathLineTo(x=100, y=100),\n    ],\n    line_color=ap.Color("#0af"),\n    line_thickness=5,\n)\n\nright_path: ap.Path = ap.Path(\n    path_data_list=[\n        ap.PathMoveTo(x=175, y=50),\n        ap.PathLineTo(x=150, y=100),\n        ap.PathLineTo(x=200, y=100),\n        ap.PathClose(),\n    ],\n    line_color=ap.Color("#0af"),\n    line_thickness=5,\n)\n\nap.save_overall_html(dest_dir_path="path_close_basic_usage/")\n```',  # noqa
    ##################################################
    "## PathClose class constructor API": "## PathClose クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Path data class for the SVG's `close path` (Z).<hr>": "SVGのパスを閉じる指定（Z）のためのパスデータのクラスです。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=3)\n>>> path: ap.Path = sprite.graphics.draw_path(\n...     path_data_list=[\n...         ap.PathMoveTo(x=0, y=00),\n...         ap.PathLineTo(x=50, y=0),\n...         ap.PathLineTo(x=50, y=50),\n...         ap.PathClose(),\n...     ]\n... )\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=3)\n>>> path: ap.Path = sprite.graphics.draw_path(\n...     path_data_list=[\n...         ap.PathMoveTo(x=0, y=00),\n...         ap.PathLineTo(x=50, y=0),\n...         ap.PathLineTo(x=50, y=50),\n...         ap.PathClose(),\n...     ]\n... )\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [Path class](https://simon-ritchie.github.io/apysc/en/path.html)": "- [Path クラス](https://simon-ritchie.github.io/apysc/jp/jp_path.html)",  # noqa
    ##################################################
    "- [Graphics draw_path interface](https://simon-ritchie.github.io/apysc/en/graphics_draw_path.html)": "- [Graphics クラスの draw_path インターフェイス](https://simon-ritchie.github.io/apysc/jp/jp_graphics_draw_path.html)",  # noqa
}



"""The class implementation for the outlined `list_alt` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialListAltOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `list_alt` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M11 7h6v2h-6zm0 4h6v2h-6zm0 4h6v2h-6zM7 7h2v2H7zm0 4h2v2H7zm0 4h2v2H7zM20.1 3H3.9c-.5 0-.9.4-.9.9v16.2c0 .4.4.9.9.9h16.2c.4 0 .9-.5.9-.9V3.9c0-.5-.5-.9-.9-.9zM19 19H5V5h14v14z"/></svg>'  # noqa



"""The class implementation for the `screen_share` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialScreenShareIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `screen_share` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M20 18c1.1 0 1.99-.9 1.99-2L22 6c0-1.11-.9-2-2-2H4c-1.11 0-2 .89-2 2v10c0 1.1.89 2 2 2H0v2h24v-2h-4zm-7-3.53v-2.19c-2.78 0-4.61.85-6 2.72.56-2.67 2.11-5.33 6-5.87V7l4 3.73-4 3.74z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: string_zfill.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# String class zfill method": "# String クラスの zfill メソッド",
    ##################################################
    "This page explains the `String` class `zfill` method.": "このページでは`String`クラスの`zfill`メソッドについて説明します。",  # noqa
    ##################################################
    "## What method is this?": "## メソッド概要",
    ##################################################
    "The `zfill` method returns a zero-filled string with the specified widgth (number of total characters).": "`zfill`メソッドは指定された文字数でゼロ埋めされた文字列を返却します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `zfill` method required the `width` argument (`int` or `ap.Int` type).": "`zfill`メソッドは`width`引数を必要とします（`int`もしくは`ap.Int`型となります）。",  # noqa
    ##################################################
    "It determines the number of total characters.": "その設定によって合計の文字数が決定します。",
    ##################################################
    "A return value becomes copied `ap.String` value.": "返却値はコピーされた`ap.String`型の値となります。",  # noqa
    ##################################################
    '```py\n# runnable\n\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nstring: ap.String = ap.String(value="1")\nstring = string.zfill(width=1)\nap.assert_equal(string, "1")\n\nstring = string.zfill(width=3)\nap.assert_equal(string, "001")\n\nstring = string.zfill(width=5)\nap.assert_equal(string, "00001")\n\nap.save_overall_html(dest_dir_path="string_zfill_basic_usage/")\n```': '```py\n# runnable\n\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nstring: ap.String = ap.String(value="1")\nstring = string.zfill(width=1)\nap.assert_equal(string, "1")\n\nstring = string.zfill(width=3)\nap.assert_equal(string, "001")\n\nstring = string.zfill(width=5)\nap.assert_equal(string, "00001")\n\nap.save_overall_html(dest_dir_path="string_zfill_basic_usage/")\n```',  # noqa
    ##################################################
    "## zfill method API": "## zfill メソッドのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Return a copy of the string left filled with 0.<hr>": "文字列の左側を0で埋めた文字列を返却します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    '- `width`: Union[int, "Int"]': '- `width`: Union[int, "Int"]',
    ##################################################
    "  - A width (length) of the string.": "  - 文字列の幅（長さ）。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `result`: String": "- `result`: String",
    ##################################################
    "  - A result string.": "  - 結果の文字列。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> _: ap.Stage = ap.Stage()\n>>> string: ap.String = ap.String("1")\n>>> string = string.zfill(width=1)\n>>> string\nString("1")\n\n>>> string = string.zfill(width=3)\n>>> string\nString("001")\n\n>>> string = string.zfill(width=5)\n>>> string\nString("00001")\n```': '```py\n>>> import apysc as ap\n>>> _: ap.Stage = ap.Stage()\n>>> string: ap.String = ap.String("1")\n>>> string = string.zfill(width=1)\n>>> string\nString("1")\n\n>>> string = string.zfill(width=3)\n>>> string\nString("001")\n\n>>> string = string.zfill(width=5)\n>>> string\nString("00001")\n```',  # noqa
}



"""The class implementation for the outlined `stars` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialStarsOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `stars` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M11.99 2C6.47 2 2 6.48 2 12s4.47 10 9.99 10C17.52 22 22 17.52 22 12S17.52 2 11.99 2zm7.48 7.16l-5.01-.43-2-4.71c3.21.19 5.91 2.27 7.01 5.14zm-5.07 6.26L12 13.98l-2.39 1.44.63-2.72-2.11-1.83 2.78-.24L12 8.06l1.09 2.56 2.78.24-2.11 1.83.64 2.73zm-2.86-11.4l-2 4.72-5.02.43c1.1-2.88 3.8-4.97 7.02-5.15zM4 12c0-.64.08-1.26.23-1.86l3.79 3.28-1.11 4.75C5.13 16.7 4 14.48 4 12zm3.84 6.82L12 16.31l4.16 2.5c-1.22.75-2.64 1.19-4.17 1.19-1.52 0-2.94-.44-4.15-1.18zm9.25-.65l-1.11-4.75 3.79-3.28c.14.59.23 1.22.23 1.86 0 2.48-1.14 4.7-2.91 6.17z"/></svg>'  # noqa



"""The class implementation for the outlined `room` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialRoomOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `room` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M12 2C8.13 2 5 5.13 5 9c0 5.25 7 13 7 13s7-7.75 7-13c0-3.87-3.13-7-7-7zM7 9c0-2.76 2.24-5 5-5s5 2.24 5 5c0 2.88-2.88 7.19-5 9.88C9.92 16.21 7 11.85 7 9z"/><circle cx="12" cy="9" r="2.5"/></svg>'  # noqa



"""The class implementation for the outlined `record_voice_over` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialRecordVoiceOverOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `record_voice_over` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M9 13c2.21 0 4-1.79 4-4s-1.79-4-4-4-4 1.79-4 4 1.79 4 4 4zm0-6c1.1 0 2 .9 2 2s-.9 2-2 2-2-.9-2-2 .9-2 2-2zm0 8c-2.67 0-8 1.34-8 4v2h16v-2c0-2.66-5.33-4-8-4zm-6 4c.22-.72 3.31-2 6-2 2.7 0 5.8 1.29 6 2H3zM15.08 7.05c.84 1.18.84 2.71 0 3.89l1.68 1.69c2.02-2.02 2.02-5.07 0-7.27l-1.68 1.69zM20.07 2l-1.63 1.63c2.77 3.02 2.77 7.56 0 10.74L20.07 16c3.9-3.89 3.91-9.95 0-14z"/></svg>'  # noqa
